/*******************************************************************************
  Filename:       hal_board_cfg.h
  Revised:        $Date: 2013-09-12 11:31:38 -0700 (Thu, 12 Sep 2013) $
  Revision:       $Revision: 35293 $

  Description:

  Abstract board-specific registers, addresses, & initialization for H/W based on the
  Texas Instruments CC254x (8051 core).


  Copyright 2006-2013 Texas Instruments Incorporated. All rights reserved.

  IMPORTANT: Your use of this Software is limited to those specific rights
  granted under the terms of a software license agreement between the user
  who downloaded the software, his/her employer (which must be your employer)
  and Texas Instruments Incorporated (the "License").  You may not use this
  Software unless you agree to abide by the terms of the License. The License
  limits your use, and you acknowledge, that the Software may not be modified,
  copied or distributed unless embedded on a Texas Instruments microcontroller
  or used solely and exclusively in conjunction with a Texas Instruments radio
  frequency transceiver, which is integrated into your product.  Other than for
  the foregoing purpose, you may not use, reproduce, copy, prepare derivative
  works of, modify, distribute, perform, display or sell this Software and/or
  its documentation for any purpose.

  YOU FURTHER ACKNOWLEDGE AND AGREE THAT THE SOFTWARE AND DOCUMENTATION ARE
  PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED,
  INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
  NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL
  TEXAS INSTRUMENTS OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER CONTRACT,
  NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR OTHER
  LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
  INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
  OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT
  OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
  (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.

  Should you have any questions regarding your right to use this Software,
  contact Texas Instruments Incorporated at www.TI.com.
*******************************************************************************/
#ifndef HAL_BOARD_CFG_H
#define HAL_BOARD_CFG_H

#ifdef __cplusplus
extern "C"
{
#endif


/*******************************************************************************
 * INCLUDES
 */

#include "hal_mcu.h"
#include "hal_defs.h"
#include "hal_types.h"

/*******************************************************************************
 * CONSTANTS
 */

/* Board Identifier */

#if !defined (HAL_BOARD_CC2530EB_REV17) && !defined (HAL_BOARD_CC2530EB_REV13)
  #define HAL_BOARD_CC2530EB_REV17
#endif

/* Clock Speed */

#define HAL_CPU_CLOCK_MHZ               32

/* Sleep Clock */

#define EXTERNAL_CRYSTAL_OSC            0x00  // external 32kHz XOSC
#define INTERNAL_RC_OSC                 0x80  // internal 32kHz RCOSC

// For non-USB, assume external, unless an internal crystal is explicitly indicated.
#if !defined (XOSC32K_INSTALLED) || (defined (XOSC32K_INSTALLED) && (XOSC32K_INSTALLED == TRUE))
#define OSC_32KHZ                       EXTERNAL_CRYSTAL_OSC
#else
#define OSC_32KHZ                       INTERNAL_RC_OSC
#endif

// Minimum Time for Stable External 32kHz Clock (in ms)
#define MIN_TIME_TO_STABLE_32KHZ_XOSC   400


#define ACTIVE_LOW                      !
#define ACTIVE_HIGH                     !!    /* double negation forces result to be '1' */

#define DCDC_BV                         BV(3)
#define DCDC_SBIT                       P1_3
#define DCDC_DDR                        P1DIR

/* LED Configuration */

#define HAL_NUM_LEDS                    1

#define HAL_LED_BLINK_DELAY()   st( { volatile uint32 i; for (i=0; i<0x5800; i++) { }; } )

/* 1 - Green */
#define LED1_BV                         BV(0)
#define LED1_SBIT                       P1_0
#define LED1_DDR                        P1DIR
#define LED1_POLARITY                   ACTIVE_LOW

/* 2 - Red */
#define LED2_BV                         BV(0)
#define LED2_SBIT                       P0_0
#define LED2_DDR                        P0DIR
#define LED2_POLARITY                   ACTIVE_LOW

/* Push Button Configuration */

/* Switch buttons */
#define HAL_KEY_RISING_EDGE             0
#define HAL_KEY_FALLING_EDGE            1

#define HAL_KEY_DEBOUNCE_VALUE          50

/* CPU port interrupt */
#define HAL_KEY_CPU_PORT_0_IF           P0IF
#define HAL_KEY_CPU_PORT_1_IF           P1IF
#define HAL_KEY_CPU_PORT_2_IF           P2IF

/* SW_1 is at P1.2 */
#define HAL_KEY_SW_1_PORT               P2
#define HAL_KEY_SW_1_BIT                BV(0)
#define HAL_KEY_SW_1_SEL                P2SEL
#define HAL_KEY_SW_1_DIR                P2DIR
  
//#define HAL_KEY_SW_1_IEN                IEN2  /* CPU interrupt mask register */
//#define HAL_KEY_SW_1_ICTL               P1IEN /* Port Interrupt Control register */
//#define HAL_KEY_SW_1_ICTLBIT            BV(3) /* P0IEN - P0.1 enable/disable bit */
//#define HAL_KEY_SW_1_IENBIT             BV(4) /* Mask bit for all of Port_0 */
//#define HAL_KEY_SW_1_PXIFG              P1IFG /* Interrupt flag at source */

#define HAL_KEY_SW_1_IEN                IEN2  /* CPU interrupt mask register */
#define HAL_KEY_SW_1_ICTL               P2IEN /* Port Interrupt Control register */
#define HAL_KEY_SW_1_ICTLBIT            BV(0) /* P0IEN - P0.1 enable/disable bit */
#define HAL_KEY_SW_1_IENBIT             BV(1) /* Mask bit for all of Port_0 */
#define HAL_KEY_SW_1_PXIFG              P2IFG /* Interrupt flag at source */

#define HAL_KEY_SW_1_EDGEBIT            BV(3)

/* SW_2 is at P1.3 */
//#define HAL_KEY_SW_2_PORT               P0
//#define HAL_KEY_SW_2_BIT                BV(1)
//#define HAL_KEY_SW_2_SEL                P0SEL
//#define HAL_KEY_SW_2_DIR                P0DIR

//#define HAL_KEY_SW_2_IEN                IEN1  /* CPU interrupt mask register */
//#define HAL_KEY_SW_2_ICTL               P0IEN /* Port Interrupt Control register */
//#define HAL_KEY_SW_2_ICTLBIT            BV(1) /* P0IEN - P0.1 enable/disable bit */
//#define HAL_KEY_SW_2_IENBIT             BV(5) /* Mask bit for all of Port_0 */
//#define HAL_KEY_SW_2_PXIFG              P0IFG /* Interrupt flag at source */

//#define HAL_KEY_SW_2_EDGEBIT            BV(0)
  
/* S1 */
#define PUSH2_BV                        BV(0)
#define PUSH2_SBIT                      P2_0
#define PUSH2_POLARITY                  ACTIVE_LOW

/* S2 */
//#define PUSH1_BV                        BV(1)
//#define PUSH1_SBIT                      P0_1
//#define PUSH2_POLARITY                  ACTIVE_LOW

/* OSAL NV implemented by internal flash pages. */

// Flash is partitioned into 8 banks of 32 KB or 16 pages.
#define HAL_FLASH_PAGE_PER_BANK         16

// Flash is constructed of 128 pages of 2 KB.
#define HAL_FLASH_PAGE_PHYS             2048

// SNV can use a larger logical page size to accomodate more or bigger items or extend lifetime.
#define HAL_FLASH_PAGE_SIZE             HAL_FLASH_PAGE_PHYS
#define HAL_FLASH_WORD_SIZE             4

// CODE banks get mapped into the XDATA range 8000-FFFF.
#define HAL_FLASH_PAGE_MAP              0x8000

// The last 16 bytes of the last available page are reserved for flash lock bits.
#define HAL_FLASH_LOCK_BITS             16

// NV page definitions must coincide with segment declaration in project *.xcl file.
#if defined NON_BANKED
#define HAL_NV_PAGE_END                 30
#elif defined HAL_BOARD_CC2541S
#define HAL_NV_PAGE_END                 125
#elif defined HAL_BOARD_F128
#define HAL_NV_PAGE_END                 62
#else
#define HAL_NV_PAGE_END                 126
#endif

// Re-defining Z_EXTADDR_LEN here so as not to include a Z-Stack .h file.
#define HAL_FLASH_IEEE_SIZE             8
#define HAL_FLASH_IEEE_PAGE             (HAL_NV_PAGE_END+1)
#define HAL_FLASH_IEEE_OSET             (HAL_FLASH_PAGE_SIZE - HAL_FLASH_LOCK_BITS - HAL_FLASH_IEEE_SIZE)
#define HAL_INFOP_IEEE_OSET             0xC

#define HAL_NV_PAGE_CNT                 2
#define HAL_NV_PAGE_BEG                 (HAL_NV_PAGE_END-HAL_NV_PAGE_CNT+1)

// Used by DMA macros to shift 1 to create a mask for DMA registers.
#define HAL_NV_DMA_CH                   0
#define HAL_DMA_CH_RX                   3
#define HAL_DMA_CH_TX                   4

#define HAL_NV_DMA_GET_DESC()           HAL_DMA_GET_DESC0()
#define HAL_NV_DMA_SET_ADDR(a)          HAL_DMA_SET_ADDR_DESC0((a))

/* Critical Vdd Monitoring to prevent flash damage or radio lockup. */

// Vdd/3 / Internal Reference X ENOB --> (Vdd / 3) / 1.15 X 127
#define VDD_2_0  74   // 2.0 V required to safely read/write internal flash.
#define VDD_2_7  100  // 2.7 V required for the Numonyx device.

#define VDD_MIN_RUN   VDD_2_0
#define VDD_MIN_NV   (VDD_2_0+4)  // 5% margin over minimum to survive a page erase and compaction.
#define VDD_MIN_XNV  (VDD_2_7+5)  // 5% margin over minimum to survive a page erase and compaction.

/*******************************************************************************
 * MACROS
 */

/* Cache Prefetch Control */

#define PREFETCH_ENABLE()  st( FCTL = 0x08; )
#define PREFETCH_DISABLE() st( FCTL = 0x04; )

/* Setting Clocks */

// switch to the 16MHz HSOSC and wait until it is stable
#define SET_OSC_TO_HSOSC()                                                     \
{                                                                              \
  CLKCONCMD = (CLKCONCMD & 0x80) | CLKCONCMD_16MHZ;                            \
  while ( (CLKCONSTA & ~0x80) != CLKCONCMD_16MHZ );                            \
}

// switch to the 32MHz XOSC and wait until it is stable
#define SET_OSC_TO_XOSC()                                                      \
{                                                                              \
  CLKCONCMD = (CLKCONCMD & 0x80) | CLKCONCMD_32MHZ;                            \
  while ( (CLKCONSTA & ~0x80) != CLKCONCMD_32MHZ );                            \
}

// set 32kHz OSC and wait until it is stable
#define SET_32KHZ_OSC()                                                        \
{                                                                              \
  CLKCONCMD = (CLKCONCMD & ~0x80) | OSC_32KHZ;                                 \
  while ( (CLKCONSTA & 0x80) != OSC_32KHZ );                                   \
}

#define START_HSOSC_XOSC()                                                     \
{                                                                              \
  SLEEPCMD &= ~OSC_PD;            /* start 16MHz RCOSC & 32MHz XOSC */         \
  while (!(SLEEPSTA & XOSC_STB)); /* wait for stable 32MHz XOSC */             \
}

#define STOP_HSOSC()                                                           \
{                                                                              \
  SLEEPCMD |= OSC_PD;             /* stop 16MHz RCOSC */                       \
}

/* Board Initialization */
#define HAL_BOARD_INIT()                                                       \
{                                                                              \
  /* Set to 16Mhz to set 32kHz OSC, then back to 32MHz */                      \
  START_HSOSC_XOSC();                                                          \
  SET_OSC_TO_HSOSC();                                                          \
  SET_32KHZ_OSC();                                                             \
  SET_OSC_TO_XOSC();                                                           \
  STOP_HSOSC();                                                                \
                                                                               \
  /* Enable cache prefetch mode. */                                            \
  PREFETCH_ENABLE();                                                           \
                                                                               \
  P0SEL = 0;       /* Configure Port 0 as GPIO */                              \
  P1SEL = 0xE0;    /* P1.0 - P1.4 as GPIO */                              \
  P2SEL = 0;       /* Configure Port 2 as GPIO */                              \
  PERCFG |= 0x02;  /* UART1 to alternate 2 location on P1 pins. */             \
                                                                               \
  P0DIR = 0;    /* All port 0 pins (P0.0-P0.7) as input */                     \
  P1DIR = 0xF7; /* Port 1 pins P1.3 as input (Buttons), */                     \
                /* all others as output */                                     \
  P2DIR = 0x1F; /* All port 2 pins (P2.0-P2.4) as output */                    \
                                                                               \
  P0 = 0;      /* All pins on port 0 to low */                                 \
  P1 = 0x01;   /* P1.0 is HIGH, others LOW */                                 \
  P2 = 0;      /* All pins on port 2 to low */                                 \
}

#define CS_SBIT                 P1_4
#define HAL_SPI_CS_ON()         st( CS_SBIT = 0; )
#define HAL_SPI_CS_OFF()        st( CS_SBIT = 1; )
#define HAL_SPI_INIT()                                                         \
{                                                                              \
  HAL_SPI_CS_OFF();                                                            \
                                                                               \
  U1CSR = 0; /* UART1 SPI Mode as master. */                                   \
  U1GCR = 0xE0; /* MSB First, DO high to low, positive polirity*/              \
  U1GCR |= 0x0D; /* SCK frequency = 480.5kHz (max 500kHz) */                   \
  U1BAUD = 0xEC;                                                               \
}

/* Debounce */
#define HAL_DEBOUNCE(expr)    { int i; for (i=0; i<500; i++) { if (!(expr)) i = 0; } }

/* ----------- Push Buttons ---------- */
//#define HAL_PUSH_BUTTON1()        (PUSH1_POLARITY (PUSH1_SBIT))
#define HAL_PUSH_BUTTON2()        (PUSH2_POLARITY (PUSH2_SBIT))
#define HAL_PUSH_BUTTON3()        (0)
#define HAL_PUSH_BUTTON4()        (0)
#define HAL_PUSH_BUTTON5()        (0)
#define HAL_PUSH_BUTTON6()        (0)

/* LED's */

#define HAL_TURN_OFF_LED1()       st( LED1_SBIT = LED1_POLARITY (0); )
#define HAL_TURN_OFF_LED2()       st( LED2_SBIT = LED2_POLARITY (0); )
#define HAL_TURN_OFF_LED3()       HAL_TURN_OFF_LED1()
#define HAL_TURN_OFF_LED4()       HAL_TURN_OFF_LED1()

#define HAL_TURN_ON_LED1()        st( LED1_SBIT = LED1_POLARITY (1); )
#define HAL_TURN_ON_LED2()        st( LED2_SBIT = LED2_POLARITY (1); )
#define HAL_TURN_ON_LED3()        HAL_TURN_ON_LED1()
#define HAL_TURN_ON_LED4()        HAL_TURN_ON_LED1()

#define HAL_TOGGLE_LED1()         st( if (LED1_SBIT) { LED1_SBIT = 0; } else { LED1_SBIT = 1;} )
#define HAL_TOGGLE_LED2()         st( if (LED2_SBIT) { LED2_SBIT = 0; } else { LED2_SBIT = 1;} )
#define HAL_TOGGLE_LED3()         HAL_TOGGLE_LED1()
#define HAL_TOGGLE_LED4()         HAL_TOGGLE_LED1()

#define HAL_STATE_LED1()          (LED1_POLARITY (LED1_SBIT))
#define HAL_STATE_LED2()          (LED2_POLARITY (LED2_SBIT))
#define HAL_STATE_LED3()          HAL_STATE_LED1()
#define HAL_STATE_LED4()          HAL_STATE_LED1()


/* Driver Configuration */

/* Set to TRUE enable H/W TIMER usage, FALSE disable it */
#ifndef HAL_TIMER
#define HAL_TIMER FALSE
#endif

/* Set to TRUE enable ADC usage, FALSE disable it */
#ifndef HAL_ADC
#define HAL_ADC TRUE
#endif

/* Set to TRUE enable DMA usage, FALSE disable it */
#ifndef HAL_DMA
#define HAL_DMA TRUE
#endif

/* Set to TRUE enable Flash access, FALSE disable it */
#ifndef HAL_FLASH
#define HAL_FLASH TRUE
#endif

/* Set to TRUE enable AES usage, FALSE disable it */
#ifndef HAL_AES
#define HAL_AES TRUE
#endif

#ifndef HAL_AES_DMA
#define HAL_AES_DMA TRUE
#endif

/* Set to TRUE enable LCD usage, FALSE disable it */
#ifndef HAL_LCD
#define HAL_LCD TRUE
#endif

/* Set to TRUE enable LED usage, FALSE disable it */
#ifndef HAL_LED
#define HAL_LED TRUE
#endif
#if (!defined BLINK_LEDS) && (HAL_LED == TRUE)
#define BLINK_LEDS
#endif

/* Set to TRUE enable KEY usage, FALSE disable it */
#ifndef HAL_KEY
#define HAL_KEY TRUE
#endif

/* Set to TRUE enable UART usage, FALSE disable it */
#ifndef HAL_UART
#if (defined ZAPP_P1) || (defined ZAPP_P2) || (defined ZTOOL_P1) || (defined ZTOOL_P2)
#define HAL_UART TRUE
#else
#define HAL_UART FALSE
#endif
#endif

#if HAL_UART
// Always prefer to use DMA over ISR.
#if HAL_DMA
  #ifndef HAL_UART_DMA
    #if (defined ZAPP_P1) || (defined ZTOOL_P1)
      #define HAL_UART_DMA  1
    #elif (defined ZAPP_P2) || (defined ZTOOL_P2)
      #define HAL_UART_DMA  2
    #else
      #define HAL_UART_DMA  1
    #endif
  #endif
  #ifndef HAL_UART_ISR
    #define HAL_UART_ISR  0
  #endif
#else
  #ifndef HAL_UART_ISR
    #if (defined ZAPP_P1) || (defined ZTOOL_P1)
      #define HAL_UART_ISR  1
    #elif (defined ZAPP_P2) || (defined ZTOOL_P2)
      #define HAL_UART_ISR  2
    #else
      #define HAL_UART_ISR  1
    #endif
  #endif
  #ifndef HAL_UART_DMA
    #define HAL_UART_DMA  0
  #endif
#endif

// Used to set P2 priority - USART0 over USART1 if both are defined.
#if ((HAL_UART_DMA == 1) || (HAL_UART_ISR == 1))
#define HAL_UART_PRIPO             0x00
#else
#define HAL_UART_PRIPO             0x40
#endif

#else
#define HAL_UART_DMA  0
#define HAL_UART_ISR  0
#endif

#if !defined HAL_UART_SPI
#define HAL_UART_SPI  0
#endif

#define WAIT_1US(t)                   \
        for (uint32 i = 0; i<t; i++)     \
          {                             \
            asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); \
            asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); \
            asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); \
            asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); \
            asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); \
            asm("nop"); asm("nop"); asm("nop"); asm("nop"); asm("nop"); \
            asm("nop"); asm("nop");                                     \
          }         

#ifdef __cplusplus
}
#endif

#endif /* HAL_BOARD_CFG_H */

/*******************************************************************************
*/
